import requests
import uuid

BASE_URL = "https://api.groupme.com/v3/"

class DirectMessage():
    def __init__(self, token, from_response_obj):
        self.token = token
        for key, val in from_response_obj.items():
            setattr(self, key, val)
    
    def _verify_success(self, response):
        if response.status_code > 204:
            raise Exception("Status code error: " + response.url + " returned code " + response.status_code + "with meta: " + response.json()["meta"])
    
    def get_messages(self, use_this_user_id=True, *args):
        res = requests.get(f"{BASE_URL}direct_messages?other_user_id={self.other_user.id if use_this_user_id == True else use_this_user_id}&before_id={args[0]}&since_id={args[1]}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return res.json()["response"]
    
    def attachment_obj_creator(type="image", *args):
        if type == "image":
            return {
                "type": type,
                "url": args[0]
            }
        if type == "location":
            return {
                "type": type,
                "name": args[0],
                "lat": args[1],
                "lng": args[2]
            }
        if type == "emoji":
            return {
                "type": type,
                "placeholder": args[0],
                "charmap": args[1]
            }
        if type == "mentions":
            return {
                "type": "mentions",
                "loci": args[0],
                "user_ids": args[1]
            }
        raise Exception("Bad type, must be one of: image, location, emoji, mentions")
    
    def send_message(self, text, attachments=[], use_this_user_id=True):
        res = requests.post(f"{BASE_URL}direct_messages", headers={"X-Access-Token": self.token}, json={
            "source_guid": str(uuid.uuid4()),
            "recipient_id": self.other_user.id if use_this_user_id == True else use_this_user_id,
            "text": text,
            "attachments": attachments
        })
        self._verify_success(res)
        return res.json["response"]["message"]


import requests
from group import Group
from dm import DirectMessage
from bot import Bot

BASE_URL = "https://api.groupme.com/v3/"

class User:
    def __init__(self, token):
        self.token = token
        self.group_id = 0
    
    def _verify_success(self, response):
        if response.status_code > 204:
            raise Exception("Status code error: " + response.url + " returned code " + response.status_code + "with meta: " + response.json()["meta"])
    
    def get_groups(self, page=1, per_page=10, omit_memberships=False):
        omit_str = "&omit=memberships" if omit_memberships else ""
        res = requests.get(f"{BASE_URL}groups?page={page}&per_page={per_page}{omit_str}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        groups = []
        for raw_group in res.json()["response"]:
            groups.append(Group(from_response_obj=raw_group))
        
    def get_former_groups(self):
        res = requests.get(f"{BASE_URL}groups/former", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        groups = []
        for raw_group in res.json()["response"]:
            groups.append(Group(self.token, from_response_obj=raw_group))
    
    def get_group(self, group_id):
        res = requests.get(f"{BASE_URL}group/{group_id}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return Group(self.token, from_response_obj=res.json()["response"])
    
    def create_group(self, name, desc="", image_url="", share=False):
        assert name is not ""
        res = requests.post(f"{BASE_URL}group/", headers={"X-Access-Token": self.token}, json={"name": name, "desc": desc, "image_url": image_url, "share": share})
        self._verify_success(res)
        return Group(self.token, from_response_obj=res.json()["response"])
    
    def update_group(self, group_id, name="", desc="", image_url="", office_mode="", share=False):
        res = requests.post(f"{BASE_URL}group/{group_id}/update", headers={"X-Access-Token": self.token}, json={"name": name, "desc": desc, "image_url": image_url, "office_mode": office_mode, "share": share})
        self._verify_success(res)
        return Group(self.token, from_response_obj=res.json()["response"])
    
    def destroy_group(self, group_id):
        res = requests.post(f"{BASE_URL}group/{group_id}/destroy", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return True
    
    def join_group(self, group_id, share_token):
        res = requests.post(f"{BASE_URL}groups/{group_id}/join/{share_token}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return True
    
    def rejoin_former_group(self, group_id):
        res = requests.post(f"{BASE_URL}groups/join", headers={"X-Access-Token": self.token}, json={"group_id": group_id})
        self._verify_success(res)
        return True
    
    def change_owner(self, group_id, new_owner_user_id):
        res = requests.post(f"{BASE_URL}groups/change_owners", headers={"X-Access-Token": self.token}, json={
            "requests": [
                {
                    "group_id": group_id,
                    "owner_id": new_owner_user_id
                }
            ]
        })
        self._verify_success(res)
        return res.json()["response"]["results"][0]["status"]
    
    def get_dms(self, page=1, per_page=10):
        res = requests.post(f"{BASE_URL}chats", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        dms = []
        for raw_dm in res.json()["response"]:
            dms.append(DirectMessage(from_response_obj=raw_dm))
        return dms
    
    def like_message(self, conversation_id, message_id):
        res = requests.post(f"{BASE_URL}messages/{conversation_id}/{message_id}/like", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return True
    
    def unlike_message(self, conversation_id, message_id):
        res = requests.post(f"{BASE_URL}messages/{conversation_id}/{message_id}/unlike", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return True
    
    def create_bot(self, name, group_id, avatar_url="", callback_url="", dm_notification=False):
        #idk if this will work, but if you want the bot to recive dms for this user, set group_id to an empty string and set dm_notification to True
        res = requests.post(f"{BASE_URL}bots", headers={"X-Access-Token": self.token}, json={
            "bot": {
                "name": name,
                "group_id": group_id,
                "avatar_url": avatar_url,
                "callback_url": callback_url,
                "dm_notification": dm_notification
            }
        })
        self._verify_success(res)
        return Bot(self.token, from_response_obj=res.json()["response"])

    def get_bots(self):
        res = requests.get(f"{BASE_URL}bots", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        bots = []
        for raw_bot in res.json()["response"]:
            bots.append(Bot(from_response_obj=raw_bot))
        return bots
    
    def delete_bot(self, bot_id):
        res = requests.post(f"{BASE_URL}bots/destroy", headers={"X-Access-Token": self.token}, json={"bot_id": bot_id})
        self._verify_success(res)
        return res.json()["response"]
    
    def get_my_user_info(self):
        res = requests.get(f"{BASE_URL}/users/me", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return res.json["response"]
    
    def update_my_user_info(self, avatar_url="", name="", email="", zip_code=""):
        req_obj = {}
        if avatar_url != "": req_obj["avatar_url"] = avatar_url
        if name != "": req_obj["name"] = name
        if email != "": req_obj["email"] = email
        if zip_code != "": req_obj["zip_code"] = zip_code
        res = requests.post(f"{BASE_URL}users/update", headers={"X-Access-Token": self.token}, json=req_obj)
        self._verify_success(res)
        return res.json["response"]
    
    def enable_sms_mode(self, duration, registration_id=""):
        res = requests.post(f"{BASE_URL}users/sms_mode", headers={"X-Access-Token": self.token}, json={
            "duration": duration,
            "registration_id": registration_id
        })
        self._verify_success(res)
        return True

    def disable_sms_mode(self):
        res = requests.post(f"{BASE_URL}users/sms_mode/delete", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return True
    
    def get_blocks(self):
        res = requests.get(f"{BASE_URL}blocks?user={self.get_my_user_info()['id']}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return res.json()["response"]
    
    def does_block_exist(self, other_user_id):
        res = requests.get(f"{BASE_URL}blocks/between?user={self.get_my_user_info()['id']}&otherUser={other_user_id}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return res.json()["response"]["between"]
    
    def block_user(self, other_user_id):
        res = requests.post(f"{BASE_URL}blocks?user={self.get_my_user_info()['id']}&otherUser={other_user_id}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return res.json()["response"]
    
    def unblock_user(self, other_user_id):
        res = requests.post(f"{BASE_URL}blocks/delete?user={self.get_my_user_info()['id']}&otherUser={other_user_id}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return True


import requests
import uuid

BASE_URL = "https://api.groupme.com/v3/"

class Group():
    def __init__(self, token, id=0, from_response_obj=False):
        self.token = token
        if not from_response_obj:
            self.id = id
        else:
            for key, val in from_response_obj.items():
                setattr(self, key, val)
    
    def _verify_success(self, response):
        if response.status_code > 204:
            raise Exception("Status code error: " + response.url + " returned code " + response.status_code + "with meta: " + response.json()["meta"])
    
    def add_members(self, members_to_add):
        res = requests.post(f"{BASE_URL}groups/{self.id}/members/add", headers={"X-Access-Token": self.token}, json={
            "members": members_to_add
        })
        self._verify_success(res)
        return res.json()["response"]["results_id"]
    
    def get_member_add_results(self, results_id):
        res = requests.get(f"{BASE_URL}groups/{self.id}/members/results/{results_id}", headers={"X-Access-Token": self.token})
        if res.status_code == 503 or res.status_code == 404: return False
        self._verify_success(res)
        return res.json()["response"]["members"]
    
    def remove_member(self, membership_id):
        res = requests.post(f"{BASE_URL}groups/{self.id}/members/{membership_id}/remove", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return True
    
    def update_my_nickname(self, new_nickname):
        res = requests.post(f"{BASE_URL}groups/{self.id}/memberships/update", headers={"X-Access-Token": self.token}, json={
            "membership": {
                "nickname": new_nickname
            }
        })
        self._verify_success(res)
        return True
    
    def get_messages(self, before_id=0, since_id=0, after_id=0, limit=20):
        req_obj = {}
        if before_id != 0: req_obj["before_id"] = before_id
        if after_id != 0: req_obj["after_id"] = after_id
        if since_id != 0: req_obj["since_id"] = since_id
        req_obj["limit"] = limit
        res = requests.get(f"{BASE_URL}groups/{self.id}/messages", headers={"X-Access-Token": self.token}, json=req_obj)
        self._verify_success(res)
        return res.json["response"]["messages"]
    
    def attachment_obj_creator(type="image", *args):
        if type == "image":
            return {
                "type": type,
                "url": args[0]
            }
        if type == "location":
            return {
                "type": type,
                "name": args[0],
                "lat": args[1],
                "lng": args[2]
            }
        if type == "emoji":
            return {
                "type": type,
                "placeholder": args[0],
                "charmap": args[1]
            }
        if type == "mentions":
            return {
                "type": "mentions",
                "loci": args[0],
                "user_ids": args[1]
            }
        raise Exception("Bad type, must be one of: image, location, emoji, mentions")

    def send_message(self, text, attachments=[]):
        res = requests.post(f"{BASE_URL}groups/{self.id}/messages", headers={"X-Access-Token": self.token}, json={
            "message": {
                "source_guid": str(uuid.uuid4()),
                "text": text,
                "attachments": attachments
            }
        })
        self._verify_success(res)
        return res.json["response"]["message"]
    
    def get_liked_messages(self, time_period):
        res = requests.get(f"{BASE_URL}groups/{self.id}/likes?period={time_period}", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return res.json["response"]["messages"]
    
    def get_my_liked_messages(self):
        res = requests.get(f"{BASE_URL}groups/{self.id}/likes/mine", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return res.json["response"]["messages"]
    
    def get_my_messages_others_have_liked(self):
        res = requests.get(f"{BASE_URL}groups/{self.id}/likes/for_me", headers={"X-Access-Token": self.token})
        self._verify_success(res)
        return res.json["response"]["messages"]


import requests

BASE_URL = "https://api.groupme.com/v3/"

class Bot:
    def __init__(self, token, bot_id=0, from_response_obj=False):
        self.token = token
        if not from_response_obj:
            self.bot_id = bot_id
        else:
            for key, val in from_response_obj.items():
                setattr(self, key, val)
    
    def _verify_success(self, response):
        if response.status_code > 204:
            raise Exception("Status code error: " + response.url + " returned code " + response.status_code + "with meta: " + response.json()["meta"])
    
    def send_message(self, text, picture_url=""):
        res = requests.post(f"{BASE_URL}bots/post", headers={"X-Access-Token": self.token}, json={
            "bot_id": self.bot_id,
            "text": text,
            "picture_url": picture_url
        })
        self._verify_success(res)
        return res.json["response"]["message"]
